@@ -23,6 +23,8 @@ extern enum cgi_method_t cgi_method;
 
 extern char *cgi_error(void);
 extern cgidata_t *cgi_request(void);
+extern char *csp_header(const char *pagename); 
+extern int cgi_refererok(char *expected); 
 extern char *get_cookie(char *cookiename);
 
 #endif
